require 'spec_helper'
require 'ostruct'

describe Rainforest::Client do
  # Should keep this API key for tets only
  let(:api_key) { '1f98f025af50d0d6732e79625abb5a06'}
  subject do
    Rainforest::Client.new
  end
  
  it "should create a new instance" do
    subject.should be_an_instance_of Rainforest::Client
  end

  it 'should update API key' do
    Rainforest::Client.api_key = api_key
    Rainforest::Client.api_key.should == api_key
  end

  describe '#build_request_uri' do
    it "should throw error with nil resource" do
      expect { subject.build_request_uri}.to raise_error ArgumentError
    end
    
    it "should throw error with empty resource" do
      expect { subject.build_request_uri('  ')}.to raise_error ArgumentError
    end

    it "should throw error with invalid resource" do
      expect { subject.build_request_uri('unknow resource')}.to raise_error ArgumentError
    end

    it "build correct end point" do
      subject.build_request_uri('runs').should == 'https://app.rainforestqa.com/api/1/runs'
    end
  end

  describe '#request' do
    context "with invalid param" do 
      it "should raise error without an API token" do 
        Rainforest::Client.api_key = nil
        expect { subject.request('runs', {}, :get)}.to raise_error Rainforest::ConfigurationError
      end
      it "should raise error with input argument" do 
        Rainforest::Client.api_key = api_key
        expect { subject.request('lopruns', {}, :invalid)}.to raise_error ArgumentError
      end
    end
    
    context "with valid param" do 
      before(:each) do 
        Rainforest::Client.api_key = api_key
      end

      it "should retrieve all runs" do
        payload = {tests: [2286]}
        uri = subject.build_request_uri('runs')
        HTTParty.stub(:get).with(uri, {body: "{}", headers: {"CLIENT_TOKEN" => api_key, "Content-type" => "application/json"}}) do 
          OpenStruct.new({body: '[{"id":4337,"object":"Run","created_at":"2013-11-16T08:30:36Z","environment_id":1235,"test_count":1,"state_log":[{"created_at":"2013-11-16T08:30:46Z","event":"validate_run!","from":"queued","id":11846764,"state_column":"state","stateful_id":4337,"stateful_type":"Run","to":"validating","transitioned_at":"2013-11-16T08:30:46Z","updated_at":"2013-11-16T08:30:46Z"},{"created_at":"2013-11-16T08:30:46Z","event":"validated!","from":"validating","id":11846763,"state_column":"state","stateful_id":4337,"stateful_type":"Run","to":"in_progress","transitioned_at":"2013-11-16T08:30:46Z","updated_at":"2013-11-16T08:30:46Z"}],"state":"in_progress","result":"no_result","expected_wait_time":5400.0,"browsers":[{"name":"chrome","state":"enabled"},{"name":"firefox","state":"enabled"},{"name":"ie8","state":"enabled"},{"name":"ie9","state":"enabled"},{"name":"safari","state":"enabled"}],"requested_tests":[2286]},{"id":4270,"object":"Run","created_at":"2013-11-14T05:47:46Z","environment_id":1235,"test_count":1,"state_log":[{"created_at":"2013-11-14T06:06:54Z","event":"complete!","from":"in_progress","id":11780317,"state_column":"state","stateful_id":4270,"stateful_type":"Run","to":"complete","transitioned_at":"2013-11-14T06:06:54Z","updated_at":"2013-11-14T06:06:54Z"},{"created_at":"2013-11-14T06:06:44Z","event":"failed!","from":"no_result","id":11780303,"state_column":"result","stateful_id":4270,"stateful_type":"Run","to":"failed","transitioned_at":"2013-11-14T06:06:44Z","updated_at":"2013-11-14T06:06:44Z"},{"created_at":"2013-11-14T06:06:43Z","event":"failed!","from":"no_result","id":11780301,"state_column":"result","stateful_id":4270,"stateful_type":"Run","to":"failed","transitioned_at":"2013-11-14T06:06:43Z","updated_at":"2013-11-14T06:06:43Z"},{"created_at":"2013-11-14T05:48:05Z","event":"validate_run!","from":"queued","id":11779865,"state_column":"state","stateful_id":4270,"stateful_type":"Run","to":"validating","transitioned_at":"2013-11-14T05:48:05Z","updated_at":"2013-11-14T05:48:05Z"},{"created_at":"2013-11-14T05:48:05Z","event":"validated!","from":"validating","id":11779864,"state_column":"state","stateful_id":4270,"stateful_type":"Run","to":"in_progress","transitioned_at":"2013-11-14T05:48:05Z","updated_at":"2013-11-14T05:48:05Z"}],"state":"complete","result":"failed","expected_wait_time":5400.0,"browsers":[{"name":"chrome","state":"enabled"},{"name":"firefox","state":"enabled"},{"name":"ie8","state":"enabled"},{"name":"ie9","state":"enabled"},{"name":"safari","state":"enabled"}],"requested_tests":[2286]},{"id":4269,"object":"Run","created_at":"2013-11-14T05:30:51Z","environment_id":1235,"test_count":1,"state_log":[{"created_at":"2013-11-14T05:54:29Z","event":"complete!","from":"in_progress","id":11780079,"state_column":"state","stateful_id":4269,"stateful_type":"Run","to":"complete","transitioned_at":"2013-11-14T05:54:29Z","updated_at":"2013-11-14T05:54:29Z"},{"created_at":"2013-11-14T05:48:01Z","event":"failed!","from":"failed","id":11779840,"state_column":"result","stateful_id":4269,"stateful_type":"Run","to":"failed","transitioned_at":"2013-11-14T05:48:01Z","updated_at":"2013-11-14T05:48:01Z"},{"created_at":"2013-11-14T05:36:19Z","event":"failed!","from":"no_result","id":11779597,"state_column":"result","stateful_id":4269,"stateful_type":"Run","to":"failed","transitioned_at":"2013-11-14T05:36:19Z","updated_at":"2013-11-14T05:36:19Z"},{"created_at":"2013-11-14T05:36:18Z","event":"failed!","from":"no_result","id":11779595,"state_column":"result","stateful_id":4269,"stateful_type":"Run","to":"failed","transitioned_at":"2013-11-14T05:36:18Z","updated_at":"2013-11-14T05:36:18Z"},{"created_at":"2013-11-14T05:31:05Z","event":"validate_run!","from":"queued","id":11779400,"state_column":"state","stateful_id":4269,"stateful_type":"Run","to":"validating","transitioned_at":"2013-11-14T05:31:05Z","updated_at":"2013-11-14T05:31:05Z"},{"created_at":"2013-11-14T05:31:05Z","event":"validated!","from":"validating","id":11779399,"state_column":"state","stateful_id":4269,"stateful_type":"Run","to":"in_progress","transitioned_at":"2013-11-14T05:31:05Z","updated_at":"2013-11-14T05:31:05Z"}],"state":"complete","result":"failed","expected_wait_time":5400.0,"browsers":[{"name":"chrome","state":"enabled"},{"name":"firefox","state":"enabled"},{"name":"ie8","state":"enabled"},{"name":"ie9","state":"enabled"},{"name":"safari","state":"enabled"}],"requested_tests":[2285]}]'})
        end
        runs = subject.request('runs', {}, :get)
        expect(runs.count).to eq(3)
        expect(runs[0]['id']).to eq(4337)
        expect(runs[0]['object']).to eq('Run')
      end

      it "should create a run" do
        payload = {tests: [2286]}
        uri = subject.build_request_uri('runs')
        HTTParty.stub(:post).with(uri, {body: payload.to_json, headers: {"CLIENT_TOKEN" => api_key, "Content-type" => "application/json"}}) do 
          OpenStruct.new({:body=> '{"id":4338,"object":"Run","created_at":"2013-11-16T09:03:19Z","environment_id":1235,"test_count":0,"state_log":[],"state":"queued","result":"no_result","expected_wait_time":5400.0,"browsers":[{"name":"chrome","state":"disabled"},{"name":"firefox","state":"disabled"},{"name":"ie8","state":"disabled"},{"name":"ie9","state":"disabled"},{"name":"safari","state":"disabled"}],"requested_tests":[2286]}'})
        end
        runs = subject.request('runs', payload, :post)
        expect(runs['id']).to eq(4338)
        expect(runs['object']).to eq('Run')
      end

      it "should abort a run" do
        run_id = 4338
        payload = {:run => run_id}
        uri = subject.build_request_uri('runs/4338')
        HTTParty.stub(:delete).with(uri, {body: payload.to_json, headers: {"CLIENT_TOKEN" => api_key, "Content-type" => "application/json"}}) do 
          OpenStruct.new({:body=> '{"id":4338,"object":"Run","created_at":"2013-11-16T09:03:19Z","environment_id":1235,"test_count":1,"state_log":[{"created_at":"2013-11-16T09:14:13Z","event":"abort!","from":"in_progress","id":11847319,"state_column":"state","stateful_id":4338,"stateful_type":"Run","to":"aborted","transitioned_at":"2013-11-16T09:14:13Z","updated_at":"2013-11-16T09:14:13Z"},{"created_at":"2013-11-16T09:03:39Z","event":"validate_run!","from":"queued","id":11847142,"state_column":"state","stateful_id":4338,"stateful_type":"Run","to":"validating","transitioned_at":"2013-11-16T09:03:39Z","updated_at":"2013-11-16T09:03:39Z"},{"created_at":"2013-11-16T09:03:39Z","event":"validated!","from":"validating","id":11847141,"state_column":"state","stateful_id":4338,"stateful_type":"Run","to":"in_progress","transitioned_at":"2013-11-16T09:03:39Z","updated_at":"2013-11-16T09:03:39Z"}],"state":"aborted","result":"no_result","expected_wait_time":5400.0,"browsers":[{"name":"chrome","state":"enabled"},{"name":"firefox","state":"enabled"},{"name":"ie8","state":"enabled"},{"name":"ie9","state":"enabled"},{"name":"safari","state":"enabled"}],"requested_tests":[2286]}'})
        end
        runs = subject.request('runs/4338', payload, :delete)
        expect(runs['id']).to eq(4338)
        expect(runs['state']).to eq('aborted')
        expect(runs['state_log'][0]['to']).to eq('aborted')
      end
    end
  end

end